#pragma once

#include <memory>

#include <QObject>

#include <RobotAPI/interface/armem/query.h>


class QCheckBox;
class QComboBox;
class QDateTimeEdit;
class QLabel;
class QSpinBox;
class QTabWidget;


namespace armarx::armem
{

    struct EntityForm : QObject
    {
        Q_OBJECT

    public:

        EntityForm();

        virtual query::EntityQueryPtr makeQuery() = 0;
        void setDisplayFormat(QDateTimeEdit* dt);

    signals:
        void queryChanged();

    public:

        QWidget* widget;
    };

    struct EntityFormAll : public EntityForm
    {
        query::EntityQueryPtr makeQuery() override;
    };
    struct EntityFormSingle : public EntityForm
    {
        EntityFormSingle();

        QLabel* label;
        QDateTimeEdit* dateTime;
        QCheckBox* latest;

        query::EntityQueryPtr makeQuery() override;
    };
    struct EntityFormTimeRange : public EntityForm
    {
        EntityFormTimeRange();

        QLabel* fromLabel;
        QDateTimeEdit* fromDateTime;
        QCheckBox* fromBegin;

        QLabel* toLabel;
        QDateTimeEdit* toDateTime;
        QCheckBox* toEnd;

        query::EntityQueryPtr makeQuery() override;
    };
    struct EntityFormIndexRange : public EntityForm
    {
        EntityFormIndexRange();

        QLabel* firstLabel;
        QSpinBox* firstSpinBox;
        QCheckBox* firstBegin;

        QLabel* lastLabel;
        QSpinBox* lastSpinBox;
        QCheckBox* lastEnd;

        query::EntityQueryPtr makeQuery() override;
    };


    class EntityQueryWidget : public QObject
    {
        Q_OBJECT

        using This = EntityQueryWidget;

    public:

        EntityQueryWidget();


    public slots:

    signals:

        void queryChanged();


    private slots:

        void updateQuery();
        void updateVisibleForm(QString visible);


    signals:

        void queryOutdated();


    private:

        armem::query::EntityQueryPtr makeQuery();
        void hideAllForms();


    public:

        QWidget* widget;
        armem::query::EntityQueryPtr query;

        QComboBox* queryComboBox;
        QCheckBox* dataCheckBox;

        std::map<QString, std::unique_ptr<EntityForm>> queryForms;

    };


    class MemoryQueryWidget : public QObject
    {
        Q_OBJECT
        using This = MemoryQueryWidget;

    public:

        MemoryQueryWidget();

        void setWidget(QTabWidget* tabWidget);

        armem::query::EntityQueryPtr entityQuery();


    public slots:

        void initWidget();


    signals:

        void widgetSet();
        void entityQueryChanged(armem::query::EntityQueryPtr query);


    private slots:

    signals:


    private:

        armem::query::EntityQueryPtr makeEntityQuery();



    private:

        QTabWidget* widget = nullptr;

        std::unique_ptr<EntityQueryWidget> entityWidget;



    };

}
