#pragma once

#include <functional>
#include <map>
#include <sstream>

#include <QTreeWidget>


#include <RobotAPI/libraries/armem/memory/Memory.h>
#include <RobotAPI/gui-plugins/ObjectPoseGui/TreeWidgetBuilder.h>


namespace armarx::armem
{

    /**
     * A class to efficiently build and maintain sorted items of `QTreeWidget`
     * or `QTreeWidgetItem` based on a sorted container matching the intended structure.
     */
    class MemoryTreeWidgetBuilder
    {
    public:

        MemoryTreeWidgetBuilder();


        void updateChildren(const armem::Memory& memory, QTreeWidget* tree);
        void updateChildren(const std::map<std::string, const armem::Memory*>& memories, QTreeWidget* tree);

        void updateChildren(const armem::Memory& memory, QTreeWidgetItem* memoryItem);
        void updateChildren(const armem::CoreSegment& coreSeg, QTreeWidgetItem* coreSegItem);
        void updateChildren(const armem::ProviderSegment& provSeg, QTreeWidgetItem* provSegItem);
        void updateChildren(const armem::Entity& entity, QTreeWidgetItem* entityItem);
        void updateChildren(const armem::EntitySnapshot& snapshot, QTreeWidgetItem* snapshotItem);
        void updateChildren(const armem::EntityData& data, QTreeWidgetItem* parent);


        QTreeWidgetItem* makeItem(const string& key, const armem::detail::MemoryLevel* container);
        QTreeWidgetItem* makeItem(const string& key, const armem::detail::MemoryContainer* container);
        void updateItem(const armem::detail::MemoryLevel* level, QTreeWidgetItem* item);
        void updateItem(const armem::detail::MemoryContainer* container, QTreeWidgetItem* item);


    public:

        MapTreeWidgetBuilder<std::string, const Memory*> memoryBuilder;
        MapTreeWidgetBuilder<std::string, CoreSegmentPtr> coreSegmentBuilder;
        MapTreeWidgetBuilder<std::string, ProviderSegmentPtr> provSegmentBuilder;
        MapTreeWidgetBuilder<std::string, EntityPtr> entityBuilder;
        MapTreeWidgetBuilder<armem::Time, EntitySnapshotPtr> snapshotBuilder;
        TreeWidgetBuilder<std::vector<EntityDataPtr>> instanceBuilder;

    };

}
