#pragma once

#include <RobotAPI/interface/aron.ice>


module armarx
{
    module armem
    {
        module query
        {

            /// Which entity snapshots to get from an entity?
            class EntityQuery
            {
                /// If false, no AronData is transferred.
                bool withData = true;
            };
            sequence<EntityQuery> EntityQuerySeq;

            module entity
            {
                /// Get all snapshots.
                class All extends EntityQuery
                {
                };
                /// Get a single snapshot (default for latest).
                class Single extends EntityQuery
                {
                    long timestamp = -1;  // -1 for latest
                };
                /**
                 * @brief Get snapshots based on a time range (default for all).
                 */
                class TimeRange extends EntityQuery
                {
                    long minTimestamp = -1;  // -1 for oldest
                    long maxTimestamp = -1;  // -1 for latest
                };
                /**
                 * @brief Get snapshots based on an index range (default for all).
                 *
                 * The returned range of snapshots can be smaller than the requested
                 * range, if less snapshots exist.
                 *
                 * Uses python negative index semantics:
                 * * -1 = last item
                 * * -2 = item before last item, ...
                 */
                class IndexRange extends EntityQuery
                {
                    long first = 0;  ///< First index to get.
                    long last = -1;  ///< Last index to get (inclusive).
                };
            }


            /// Which entities to get from a provider segment?
            class ProviderSegmentQuery
            {
                EntityQuerySeq entityQueries;
            };
            sequence<ProviderSegmentQuery> ProviderSegmentQuerySeq;
            module provider
            {
                class All extends ProviderSegmentQuery
                {
                };
                class Single extends ProviderSegmentQuery
                {
                    string entityName;
                };
                class Regex extends ProviderSegmentQuery
                {
                    string entityNameRegex;
                };
            }


            /// Which provider segments to get from a core segment?
            class CoreSegmentQuery
            {
                ProviderSegmentQuerySeq providerSegmentQueries;
            };
            sequence<CoreSegmentQuery> CoreSegmentQuerySeq;

            module core
            {
                class All extends CoreSegmentQuery
                {
                };
                class Single extends CoreSegmentQuery
                {
                    string providerSegmentName;
                };
                class Regex extends CoreSegmentQuery
                {
                    string providerSegmentNameRegex;
                };
            }


            /// Which core segments to get from a memory?
            class MemoryQuery
            {
                CoreSegmentQuerySeq coreSegmentQueries;
            };
            sequence<MemoryQuery> MemoryQuerySeq;

            module memory
            {
                class All extends MemoryQuery
                {
                };
                class Single extends MemoryQuery
                {
                    string coreSegmentName;
                };
                class Regex extends MemoryQuery
                {
                    string coreSegmentNameRegex;
                };
            }
        }

    };
};
