#include "MemoryReader.h"

#include <ArmarXCore/core/logging/Logging.h>

#include "../memory/ice_conversions.h"


namespace armarx::armem
{

    MemoryReader::MemoryReader(ReadingInterfacePrx memory) : memory(memory)
    {
    }


    MemoryPtr MemoryReader::getLatestSnapshots()
    {
        armem::query::EntityQueryPtr entityQuery = new armem::query::entity::Single();
        armem::query::ProviderSegmentQueryPtr providerQuery = new armem::query::provider::All({entityQuery});
        armem::query::CoreSegmentQueryPtr coreQuery = new armem::query::core::All({providerQuery});
        armem::query::MemoryQueryPtr memoryQuery = new armem::query::memory::All({coreQuery});

        return queryToCpp(memoryQuery);
    }

    MemoryPtr MemoryReader::getAllData()
    {
        armem::query::EntityQueryPtr entityQuery = new armem::query::entity::All();
        armem::query::ProviderSegmentQueryPtr providerQuery = new armem::query::provider::All({entityQuery});
        armem::query::CoreSegmentQueryPtr coreQuery = new armem::query::core::All({providerQuery});
        armem::query::MemoryQueryPtr memoryQuery = new armem::query::memory::All({coreQuery});

        return queryToCpp(memoryQuery);
    }

    data::MemoryPtr MemoryReader::query(query::MemoryQueryPtr query)
    {
        return this->query(query::MemoryQuerySeq{query});
    }

    data::MemoryPtr MemoryReader::query(query::MemoryQuerySeq queries)
    {
        try
        {
            return memory->query(queries);
        }
        catch (const Ice::ConnectionRefusedException& e)
        {
            ARMARX_ERROR << "Memory query failed.\nReason: " << e.what();
        }
        catch (const Ice::ConnectionLostException& e)
        {
            ARMARX_ERROR << "Memory query failed.\nReason: " << e.what();
        }
        catch (const Ice::NotRegisteredException& e)
        {
            ARMARX_ERROR << "Memory query failed.\nReason: " << e.what();
        }
        return nullptr;
    }

    MemoryPtr MemoryReader::queryToCpp(query::MemoryQueryPtr query)
    {
        return armem::fromIce<armem::MemoryPtr>(this->query(query));
    }

    MemoryPtr MemoryReader::queryToCpp(query::MemoryQuerySeq queries)
    {
        return armem::fromIce<armem::MemoryPtr>(this->query(queries));
    }

}
