#include "EntityData.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


namespace armarx::armem
{

    void EntityData::update(const InternalEntityUpdate& update, int index)
    {
        ARMARX_CHECK_FITS_SIZE(index, update.instancesData.size());

        this->index = index;
        setData(update.instancesData.at(size_t(index)));

        this->metadata.confidence = update.confidence;

        this->metadata.timeCreated = update.timeCreated;
        this->metadata.timeSent = update.timeSent;
        this->metadata.timeArrived = update.timeArrived;
    }

    void EntityData::setData(aron::data::AronDataPtr data)
    {
        this->data = data;
        try
        {
            this->dataNavigator = aron::datanavigator::AronDataNavigator::FromAronData(this->data);
        }
        catch (const armarx::LocalException&)
        {
            this->dataNavigator = nullptr;
        }
    }

    EntityDataPtr EntityData::copy(bool withData) const
    {
        EntityDataPtr other = std::make_unique<EntityData>();
        other->index = index;
        other->metadata = metadata;
        if (withData)
        {
            other->setData(data);
        }
        return other;
    }

}
