#include "EntitySnapshot.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "../error.h"


namespace armarx::armem
{


    EntitySnapshot::EntitySnapshot()
    {
    }

    EntitySnapshot::EntitySnapshot(Time time)
    {
        id().timestamp = time;
    }

    EntitySnapshot::EntitySnapshot(const EntitySnapshot& other)
    {
        *this = other;
    }

    EntitySnapshot& EntitySnapshot::operator=(const EntitySnapshot& other)
    {
        other._copySelf(*this);
        other._copyElements(*this, true);
        return *this;
    }


    void EntitySnapshot::update(const InternalEntityUpdate& update)
    {
        id().timestamp = update.timeCreated;

        instances.clear();
        for (int i = 0; i < int(update.instancesData.size()); ++i)
        {
            EntityDataPtr& data = instances.emplace_back(std::make_unique<EntityData>());
            data->update(update, i);
        }
    }


    EntityData& EntitySnapshot::getInstance(int index)
    {
        size_t si = size_t(index);
        if (index >= 0 && si < instances.size())
        {
            return *instances[si];
        }
        else
        {
            throw armem::error::MissingEntry("instance", std::to_string(index),
                                             getLevelName(), toDateTimeMilliSeconds(time()));
        }
    }


    EntityData& EntitySnapshot::getInstance(MemoryID id)
    {
        if (!id.hasInstanceIndex())
        {
            throw armem::error::InvalidMemoryID(id, "ID has no instance index.");
        }
        return getInstance(id.instanceIndex);
    }

    void EntitySnapshot::_copySelf(EntitySnapshot& other) const
    {
        other.time() = this->time();
    }

    void EntitySnapshot::_copyElements(EntitySnapshot& other, bool withData) const
    {
        other.instances.clear();
        other.instances.reserve(this->instances.size());
        for (const auto& data : this->instances)
        {
            other.instances.emplace_back(data->copy(withData));
        }
    }





}
