#pragma once

#include <map>
#include <memory>
#include <string>

#include "CoreSegment.h"
#include "detail/EntityContainer.h"


namespace armarx::armem
{

    class Memory;
    using MemoryPtr = std::unique_ptr<Memory>;


    /**
     * @brief Data of a memory consisting of multiple core segments.
     */
    class Memory : public detail::EntityContainer<CoreSegmentPtr, Memory>
    {
        using Base = detail::EntityContainer<CoreSegmentPtr, Memory>;

    public:

        using Base::EntityContainer;
        Memory(const std::string& name)
        {
            this->name() = name;
        }

        Memory(const Memory& other);
        Memory& operator=(const Memory& other);

        ~Memory() override;


        using Base::name;
        const std::string& name() const override;


        bool hasCoreSegment(const std::string& name) const;

        CoreSegment& getCoreSegment(const std::string& name);
        const CoreSegment& getCoreSegment(const std::string& name) const;

        using Base::getEntity;
        const Entity& getEntity(const MemoryID& id) const override;


        /**
         * @brief Add an empty core segment with the given name.
         * @param name The core segment name.
         * @param coreSegmentType The core segment type (optional).
         * @return The added core segment.
         */
        CoreSegment& addCoreSegment(const std::string& name, aron::type::AronTypePtr coreSegmentType = nullptr);
        /// Copy and insert a core segment.
        CoreSegment& addCoreSegment(const CoreSegment& coreSegment);
        /// Move and insert a core segment.
        CoreSegment& addCoreSegment(CoreSegmentPtr&& coreSegment);

        /**
         * @brief Add multiple core segments.
         * @param The core segment names.
         * @return The core segments. The contained pointers are never null.
         */
        std::vector<CoreSegment*> addCoreSegments(const std::vector<std::string>& names);

        virtual MemoryID update(const InternalEntityUpdate& update) override;
        using Base::update;


    public:

        std::map<std::string, CoreSegmentPtr> coreSegments;


        // MemoryLevel interface
    public:
        std::string getLevelName() const override
        {
            return "memory";
        }

        // MemoryContainer interface
    protected:
        ContainerT& _underlyingContainer() override
        {
            return coreSegments;
        }

        void _copyElements(Memory& other, bool withData) const override;

    };

}
